!
!	main.f90
!	ReplicationMS
!
!	Created by Oliver Will on 04.06.09.
!	Copyright 2009 Universität Bayreuth. All rights reserved.
!

PROGRAM main

	USE MOD_GLOBAL_VARIABLES
	USE MOD_AGENTSET
	USE MOD_MOVING
	USE MOD_MATCHING
	USE MOD_COOPERATION
	USE MOD_TRUST
	USE MOD_PAYOFFS
	USE MOD_LEARNING
	USE MOD_STATISTICS

	IMPLICIT NONE
	
	INTEGER :: i, count_par_rs, count_par_nh, count_par_mr
		
	ALLOCATE(Alter(number_of_agents), Match(number_of_agents))
	
	OPEN(unit = 42, file = final_results_file, action = 'write')
	OPEN(unit = 4242, file = sr_results_file, action = 'write')
	
	! run simulations with different rates of mobility
	DO count_par_mr = 1, UBOUND( different_mobility_rates, DIM = 1)
		mobility_rate = different_mobility_rates(count_par_mr)
		WRITE(*,*) "mobility rate: ", mobility_rate
		! run simulations with different numbers of neighbourhoods
		DO count_par_nh = 1, UBOUND( different_nr_of_nhs, DIM = 1)
			number_of_neighbourhoods = different_nr_of_nhs(count_par_nh)
			WRITE(*,*) "number of neighbourhoods: ", number_of_neighbourhoods
			ALLOCATE(OppCost(number_of_neighbourhoods + 1), NhSize(number_of_neighbourhoods + 1), localMaxPayoff(number_of_neighbourhoods), localBest(number_of_neighbourhoods))
			CALL reset_statistics
			! run simulations with different random seeds
			DO count_par_rs = 1, UBOUND( random_seeds, DIM = 1 )
				WRITE(*,*) "repetition: ", count_par_rs
		
				! set simulation up
				CALL reset_sr_statistics
				CALL set_random_seed(random_seeds(count_par_rs))
				CALL initialize_agents
				! make iterations
				DO i = 1, last_time_step
					time_step = i
					CALL moving
					CALL decide_on_market
					CALL matching
					CALL decide_on_cooperation
					CALL decide_on_trusting
					CALL calculate_payoffs
					CALL learn
					IF (time_step > (last_time_step - steps_to_record)) THEN
						CALL statistics
					END IF
				END DO
				
				CALL write_sr_results
				CALL deallocate_agentset
	
			END DO ! different random seeds
			CALL write_final_results
			DEALLOCATE(OppCost, NhSize, localMaxPayoff, localBest)
		END DO ! different numbers of neighbourhoods
	END DO ! different rates of mobility
	
	CLOSE(42)
	CLOSE(4242)
	
END PROGRAM main
