!
!	mod_agentset.f90
!	ReplicationMS
!
!	Created by Oliver Will on 04.06.09.
!	Copyright 2009 Universität Bayreuth. All rights reserved.
!

MODULE MOD_AGENTSET

	USE MOD_GLOBAL_VARIABLES

	IMPLICIT NONE
	
	SAVE
	
	REAL, ALLOCATABLE, DIMENSION(:) :: PropMkt			! agents' propensities to enter market
	REAL, ALLOCATABLE, DIMENSION(:) :: PropCoop			! agents' propensities to cooperate 
	REAL, ALLOCATABLE, DIMENSION(:) :: PropChkTTS		! agents' propensites to check telltale signs
	REAL, ALLOCATABLE, DIMENSION(:) :: Payoff			! agents' payoff in current time-step
	REAL, ALLOCATABLE, DIMENSION(:) :: CumPayoff		! agents' cumulated payoffs
	
	INTEGER, ALLOCATABLE, DIMENSION(:) :: Newcomer		! 1 if agent is newcomer else 0
	INTEGER, ALLOCATABLE, DIMENSION(:) :: Stranger	    ! agents that enter the market are strangers
	INTEGER, ALLOCATABLE, DIMENSION(:) :: PermAddress	! neighbourhood that agents are located in
	INTEGER, ALLOCATABLE, DIMENSION(:) :: TempAddress	! location where agents interact ( 0 if market is entered)
	INTEGER, ALLOCATABLE, DIMENSION(:) :: Coop			! does agent cooperate?
	INTEGER, ALLOCATABLE, DIMENSION(:) :: Trusts		! does agent trust?
	INTEGER, ALLOCATABLE, DIMENSION(:) :: Partner	    ! agents partner determined by the matching algorithm
	INTEGER, ALLOCATABLE, DIMENSION(:) :: TTSChecked	! did agent check telltale signs?
	
	REAL, ALLOCATABLE, DIMENSION(:) :: PropMkt_tmp			! agents' propensities to enter market (for updating)
	REAL, ALLOCATABLE, DIMENSION(:) :: PropCoop_tmp			! agents' propensities to cooperate (for updating)
	REAL, ALLOCATABLE, DIMENSION(:) :: PropChkTTS_tmp		! agents' propensites to check telltale signs (for updating)

CONTAINS

	SUBROUTINE initialize_agents
		
		INTEGER :: i
		REAL :: rand
		
		! memory allocation with given number of agents
		ALLOCATE(PropMkt(number_of_agents), PropCoop(number_of_agents), PropChkTTS(number_of_agents), Payoff(number_of_agents), CumPayoff(number_of_agents))
		ALLOCATE(Newcomer(number_of_agents), PermAddress(number_of_agents), TempAddress(number_of_agents), Stranger(number_of_agents), Partner(number_of_agents))
		ALLOCATE(Coop(number_of_agents), Trusts(number_of_agents), TTSChecked(number_of_agents))
		ALLOCATE(PropMkt_tmp(number_of_agents), PropCoop_tmp(number_of_agents), PropChkTTS_tmp(number_of_agents))
		
		DO i = 1, number_of_agents
			! initialise decision vector
			CALL random_number(PropMkt(i)) 
			CALL random_number(PropCoop(i))
			CALL random_number(PropChkTTS(i))
			
			! determine neighbourhood
			CALL random_number(rand)
			PermAddress(i) = ( INT( rand * number_of_neighbourhoods ) + 1 )
			TempAddress(i) = PermAddress(i)
			
			! initialise other status variables
			Payoff(i) = 0.0
			CumPayoff(i) = 0.0
			Newcomer(i) = 0
			Stranger(i) = 0
			
			PropCoop_tmp = PropCoop
			PropMkt_tmp = PropMkt
			PropChkTTS_tmp = PropChkTTS
			
		END DO
		
	END SUBROUTINE
	
	SUBROUTINE deallocate_agentset
		! memory deallocation 
		DEALLOCATE(PropMkt, PropCoop, PropChkTTS, Payoff, CumPayoff)
		DEALLOCATE(Newcomer, PermAddress, TempAddress, Stranger, Partner)
		DEALLOCATE(Coop, Trusts, TTSChecked)
		DEALLOCATE(PropMkt_tmp, PropCoop_tmp, PropChkTTS_tmp)
	END SUBROUTINE

END MODULE
