!
!	mod_learning.f90
!	ReplicationMS
!
!	Created by Oliver Will on 04.06.09.
!	Copyright 2009 Universität Bayreuth. All rights reserved.
!
MODULE MOD_LEARNING

	USE MOD_GLOBAL_VARIABLES
	USE MOD_AGENTSET
	
	IMPLICIT NONE
	
	INTEGER, ALLOCATABLE, DIMENSION(:) :: localBest
	REAL, ALLOCATABLE, DIMENSION(:) :: localMaxPayoff 
	
CONTAINS

	SUBROUTINE learn
	
		INTEGER :: n,i,role_model
		REAL :: rand
		
		! find the local role models and their cumulated payoffs
		DO n = 1, number_of_neighbourhoods
			! set maximum to a level that is extremely low
			localMaxPayoff(n) = -10000
			
			! find best inidividual in neighbourhood n
			DO i = 1, number_of_agents
				IF ( PermAddress(i) == n .AND. CumPayoff(i) > localMaxPayoff(n) ) THEN
					localMaxPayoff(n) = CumPayoff(i)
					localBest(n) = i
				END IF
			END DO
		END DO
		
		DO i = 1, number_of_agents
			IF ( Partner(i) > 0 ) THEN
				! i has a partner
				
				! choose role model of i's neighbourhood
				role_model = localBest(PermAddress(i))
					
				! reinforcement learning but ONLY if i is as good as the best agent in neighbourhood
				IF ( CumPayoff(role_model) <= CumPayoff(i) ) THEN
					
					! update probability to cooperate if i played PD
					IF ( Trusts(i) == 1 .AND. Trusts(Partner(i)) == 1 ) THEN
						! i played PD -> updated propensity to cooperate
						IF ( (Coop(i) == 1 .AND. Payoff(i) > 0) .OR. (Coop(i) == 0 .AND. Payoff(i) < 0 )) THEN
							PropCoop(i) = PropCoop_tmp(i) + ABS(Payoff(i)) * (1-PropCoop_tmp(i))
						ELSE
							PropCoop(i) = PropCoop_tmp(i) - ABS(Payoff(i))*PropCoop_tmp(i)
						END IF
					END IF
						
					! update probability to check telltale signs
					IF (( TTSChecked(i) == 1 .AND. Payoff(i) > 0 ) .OR. ( TTSChecked(i) == 0 .AND. Payoff(i) < 0)) THEN
						PropChkTTS(i) = PropChkTTS_tmp(i) + ABS(Payoff(i)) * (1-PropChkTTS_tmp(i))
					ELSE
						PropChkTTS(i) = PropChkTTS_tmp(i) - ABS(Payoff(i))*PropChkTTS_tmp(i)
					END IF
						
					! update probability to enter the market
					! there is a strange logic in the M&S code that prevents newcomers from reinforcement learning on market entrance
					!IF ( Newcomer(i) == 0 ) THEN
					
						IF (( TempAddress(i) == 0 .AND. Payoff(i) > 0 ) .OR. ( TempAddress(i) > 0 .AND. Payoff(i) < 0)) THEN
							PropMkt(i) = PropMkt_tmp(i) + ABS(Payoff(i)) * (1-PropMkt_tmp(i))
						ELSE
							PropMkt(i) = PropMkt_tmp(i) - ABS(Payoff(i))*PropMkt_tmp(i)
						END IF
						
					!END IF
					
				END IF ! reinforcement learning if role model or equally good
					
				! social learning if i is worse than role model and role model is not a newcomer
				IF ( CumPayoff(role_model) > CumPayoff(i) .AND. Newcomer(role_model) == 0 ) THEN
					
					! probability to cooperate
					CALL random_number(rand)
					IF ( rand > 0.5 ) THEN
						PropCoop(i) = PropCoop_tmp(role_model)
					END IF
						
					! probabilit to enter the market
					CALL random_number(rand)
					IF ( rand > 0.5 ) THEN
						PropMkt(i) = PropMkt_tmp(role_model)
					END IF
						
					! probability to check telltale signs
					CALL random_number(rand)
					IF ( rand > 0.5 ) THEN
						PropChkTTS(i) = PropChkTTS_tmp(role_model)
					END IF
					
				END IF ! social learning if worse than rm and rm not a newcomer
					
			END IF
		END DO
		
		PropMkt_tmp = PropMkt
		PropCoop_tmp = PropCoop
		PropChkTTS_tmp = PropChkTTS
		
	
	END SUBROUTINE learn

END MODULE
