!
!	mod_payoffs.f90
!	ReplicationMS
!
!	Created by Oliver Will on 04.06.09.
!	Copyright 2009 Universität Bayreuth. All rights reserved.
!
MODULE MOD_PAYOFFS

	USE MOD_GLOBAL_VARIABLES
	USE MOD_AGENTSET
	
	IMPLICIT NONE
	
	INTEGER, ALLOCATABLE, DIMENSION(:) :: NhSize
	REAL, ALLOCATABLE, DIMENSION(:) :: OppCost
	
CONTAINS

	SUBROUTINE calculate_payoffs
	
		INTEGER :: i
		
		! calculate opportunity costs
		DO i = 0, number_of_neighbourhoods
			NhSize(i+1) = COUNT(TempAddress == i)
			OppCost(i+1) = 1 - ( REAL( NhSize(i+1) - 1 ) / REAL( number_of_agents - 1 ) )
		END DO
		
		! determine payoffs
		DO i = 1, number_of_agents
			IF ( Partner(i) == 0 ) THEN
				Payoff(i) = -0.2
			ELSE
				IF ( Trusts(i) == 0 .OR. Trusts(Partner(i)) == 0 ) THEN
					! agent does not interact in PD and receives the exit payoff
					PayOff(i) = -0.2
				ELSE
					! agent interacts in PD
					IF ( Coop(i) == 1 ) THEN
						! i cooperates
						IF ( Coop(Partner(i)) == 1 ) THEN
							! partner cooperates -> R
							Payoff(i) = 0.7 - 0.5 * OppCost(TempAddress(i)+1)
						ELSE
							! partner defects -> S
							Payoff(i) = -0.5
						END IF 
					ELSE
						! i defects
						IF ( Coop(Partner(i)) == 1 ) THEN
							! partner cooperates -> T
							Payoff(i) = 1 - 0.5 * OppCost(TempAddress(i)+1)
						ELSE
							! partner defects -> P
							Payoff(i) = -0.2
						END IF 
					END IF ! i cooperates?
				END IF ! mutual trust?
			END IF ! agent has partner?
		END DO ! for all agents
		
		! calculated cumulated payoffs
		DO i = 1, number_of_agents
			CumPayoff(i) = CumPayoff(i) + Payoff(i)
		END DO
		
	END SUBROUTINE calculate_payoffs

END MODULE
